// C Source File
// Created 11/21/2010; 6:37:50 AM

//******************************************************************************
//
// Project:         Jumpman 89
//
// file name:       jm_collectables.c
//
// author:          David Randall "Ranman" 
//                  derandall99@gmail.com
//
// notes::          Jumpman was originally created by Randy Glover for the
//                  Atari 400, Commodore 64, IBM PCjr, and Apple II in 1982
//
//******************************************************************************


#include <tigcclib.h>
#include "extgraph.h"

#include "BasicTypes.h"
#include "BasicUtils.h"
#include "GrayUtils.h"
#include "FontUtils.h"

#include "jm_collectables.h"
#include "jm_graphics.h"
#include "jm_objects.h"
#include "jm_jumpman.h"
#include "jm_platforms.h"
#include "jm_moving_platforms.h"
#include "jm_scores.h"
#include "jm_levels.h"
#include "jm_enemies.h"

COLLECTABLE_OBJECT_Type collectable_list[COLLECTABLE_COUNT_MAX];
int2 collectable_attained;
int2 collectable_remaining;
int2 collectable_total;
bool collectable_trigger;

int2 bonus_point_delay;
int2 bonus_point_value;
ucoord1 bonus_point_pos;

#define MAX_SPELL_LETTERS 10
char spell_list[MAX_SPELL_LETTERS][2];
int2 spell_index;


//void DisableEnableObject(uint1 object_id, uint1 command);


void DrawCollectables(void)
{
	COLLECTABLE_OBJECT_Type* collectable = &collectable_list[0];
	int2 i;
	uint1* pSprite;
	uint1 game_piece;

	for (i = 0; i < collectable_total; i++)
	{
		if ((collectable->object->flags & FLAG_VISIBLE) && (collectable->delay == 0) && (collectable->available))
		{
			game_piece = collectable->game_piece;

			if ((game_piece == GAME_PIECE_Runaway_Bomb) && (collectable->speed == 0))
				game_piece--;

			pSprite = (uint1*)j7 + (game_piece * (1*4*3));

			GrayClipSprite8_SMASK_R(collectable->x, collectable->y, 4, pSprite + 0, pSprite + 4, pSprite + 8, PLANE_CurrentPlane0, PLANE_CurrentPlane1);
		}

		collectable++;
	}
}


void UpdateCollectables(void)
{
	COLLECTABLE_OBJECT_Type* collectable = &collectable_list[0];
	uint1 speed;
	uint1 direction;
	int2 i;
	uint1* pSprite;

	for (i = 0; i < collectable_total; i++)
	{
		pSprite = (uint1*)j7 + (collectable->game_piece * (1*4*3));

		if (collectable->game_piece == GAME_PIECE_Runaway_Bomb)
		{
			if (collectable->delay == 0)
			{
				speed = collectable->speed;
				direction = collectable->direction;

				if (speed == COLLECTABLE_MOVE_SPEED)
				{
					if (direction == DIR_Up)
					{
						collectable->y -= speed;
					}
					else if (direction == DIR_Down)
					{
						collectable->y += speed;
					}
					else if (direction == DIR_Right)
					{
						collectable->x += speed;
					}
					else if (direction == DIR_Left)
					{
						collectable->x -= speed;
					}

					if (collectable->x > 160)
					{
						if (direction == DIR_Right)
						{
							collectable->x = 0;
						}
						else // if (direction == DIR_Left)
						{
							collectable->x = 152;
						}

						collectable->delay = RandomNumber(100);
					}
					else if (collectable->y > 88)
					{
						if (direction == DIR_Up)
						{
							collectable->y = 84;
						}
						else // if (direction == DIR_Down)
						{
							collectable->y = 0;
						}

						collectable->delay = RandomNumber(100);
					}
					else
					{
						uint1 test_value = 0x18;
						uint1 bkgd_value[4];

						Sprite8Get_R(collectable->x, collectable->y + 3, 4, plane_visible_0, &bkgd_value[0]);

						if ((((test_value & (~bkgd_value[0])) == test_value) && ((test_value & (~bkgd_value[2])) == test_value)) ||
						    (((test_value & (~bkgd_value[1])) == test_value) && ((test_value & (~bkgd_value[3])) == test_value)))
						{
							if ((direction == DIR_Up) || (direction == DIR_Down))
							{
								if (RandomNumber(2) == 0)
								{
									collectable->direction = DIR_None;
									collectable->speed = COLLECTABLE_NORM_SPEED;
									collectable->delay = 0;
								}
							}
							else // if ((direction == DIR_Right) || (direction == DIR_Left))
							{
								if (RandomNumber(16) == 0)
								{
									collectable->direction = DIR_None;
									collectable->speed = COLLECTABLE_NORM_SPEED;
									collectable->delay = 0;
								}
							}
						}
					}
				}
				else
				{
					pSprite -= (1*4*3);

					if (RandomNumber(50) == 0)
					{
						collectable->speed = COLLECTABLE_MOVE_SPEED;

						if (collectable->first_move)
						{
							collectable->first_move = false;
							collectable->direction = DIR_Up + RandomNumber(2);
						}
						else
						{
							collectable->direction = DIR_Up + RandomNumber(4);
						}
					}
				}
			}
			else
			{
				collectable->delay--;
			}
		}

/*
		if ((collectable->flags & FLAG_VISIBLE) && (collectable->delay == 0))
		{
//					 char buffer[16];
//					 sprintf(buffer, "Collectable %d", collectable_count);
//					 GrayUserDrawStr(64,1, buffer, COLOR_White, 0);

			//DrawPlatformPiece(collectable->game_piece, FLAG_VA, collectable->x, collectable->y);
			//uint1* pSprite = (uint1*)j7 + (collectable->game_piece * (1*4*3));
			//uint1* pSprite = (uint1*)j7 + (9 * (1*4*3));
			GrayClipSprite8_SMASK_R(collectable->x, collectable->y, 4, pSprite + 0, pSprite + 4, pSprite + 8, PLANE_CurrentPlane0, PLANE_CurrentPlane1);
		}
*/
		collectable++;
	}
}


void CheckCollectables(void)
{
	uint1 flag_test_value = (FLAG_ACTIVE | FLAG_VISIBLE);
	int2 i;

	COLLECTABLE_OBJECT_Type* collectable = &collectable_list[0];

	for (i = 0; i < collectable_total; i++)
	{
		//if ((collectable->object->flags & FLAG_VISIBLE) || (collectable->game_piece == GAME_PIECE_Trigger))
		if (((collectable->object->flags & FLAG_ACTIVE) || (collectable->game_piece == GAME_PIECE_Trigger)) && (collectable->available))
		{
#if 0
			uint1  test_value = PixCollide8_R(
															(int2)collectable->x+4,
															(int2)collectable->y+1,
															(int2)jumpman.x_orig,
															(int2)jumpman.y_orig,
															(uint2)8,
															jumpman_test_sprite);
#else
			//BOUNDS_COLLIDE_UTIL(x0, y0, x1, y1, w0, w1, h0, h1) 
			uint1 test_value = BOUNDS_COLLIDE_UTIL(
															collectable->x+2, 
															collectable->y, 
															jumpman.x_orig+1, 
															jumpman.y_orig, 
															4, 6, 3, 8);
#endif
			//uint1  test_value = BOUNDS_COLLIDE82H(collectable->x, collectable->y+2, jumpman.x, jumpman.y, 4, 8);
//short TestCollide82h_R(short x0 asm("%d0"), short y0 asm("%d1"), short x1 asm("%d2"), short y1 asm("%d3"), unsigned short height0, unsigned short height1, const unsigned char* data0 asm("%a0"), const unsigned char* data1 asm("%a1")) __attribute__((__stkparm__)); ///< This function can handle two sprites of different heights \since 2.00 Beta 5


			if (test_value)
			{
				collectable->object->flags &= (~flag_test_value);
				
				if (collectable->game_piece != GAME_PIECE_Trigger)
				{
					game_score += 100;
					collectable->available = false;
				}


				if ((collectable->game_piece == GAME_PIECE_Bomb) || 
						(collectable->game_piece == GAME_PIECE_Runaway_Bomb) ||
						(collectable->game_piece == GAME_PIECE_Jewel))
				{
					collectable_remaining--;
					collectable_attained++;

					if ((collectable_attained % 3) == 0)
						collectable_trigger = true;

					if (collectable_attained >= current_level.collectable_required)
						level_complete = true;
				}

				if (collectable->object->flags & FLAG_SCRIPT)
				{
					if (collectable->game_piece == GAME_PIECE_Trigger)
					{
						if (jumpman.visible)
							ExecuteScript(collectable->script_id);
					}
					else
					{
						ExecuteScript(collectable->script_id);
					}
				}

				break;
			}
		}
/* //der not sure why i did this... but it messes up Now You See It... need to find which level needs this
		else if ((collectable->object->flags & FLAG_ACTIVE) == FLAG_ACTIVE)
		{
			if (BOUNDS_COLLIDE2HW(collectable->x-4, collectable->y-4, jumpman.x, jumpman.y, 16, 8, 16, 8))
			{
				collectable->object->flags |= FLAG_VISIBLE;
			}
		}
*/
		collectable++;
	}
}

void InitCollectables(void)
{
	static GAME_PIECE_Type collectable_type[4] = {GAME_PIECE_Bomb, GAME_PIECE_Runaway_Bomb, GAME_PIECE_Jewel, GAME_PIECE_Trigger};
	COLLECTABLE_OBJECT_Type* collectable = &collectable_list[0];
	OBJECT_Type* object;
	bool done = false;
	int2 pass = 0;

	collectable_total = 0;
	
	memset((void*)spell_list, 0, MAX_SPELL_LETTERS*2);
	spell_index = MAX_SPELL_LETTERS;


	memset((void*)(collectable_list), 0, sizeof(COLLECTABLE_OBJECT_Type) * COLLECTABLE_COUNT_MAX);

	while (!done)
	{
		object = FindObjectEntryByType(collectable_type[pass], false);

		if ((object) && (collectable_total < COLLECTABLE_COUNT_MAX))
		{
			collectable->object = object;
			collectable->game_piece = object->game_piece;
			collectable->x = object->x * 4;
			collectable->y = object->y;
			collectable->direction = DIR_None;
			collectable->delay = 0;
			collectable->speed = COLLECTABLE_NORM_SPEED;
			collectable->anim_step = 0;
			collectable->script_id = object->script_id;
			collectable->first_move = true;
			collectable->available = true;

			collectable++;
			collectable_total++;
		}
		else
		{
			pass++;

			if (pass >= 4)
				done = true;
		}
	}

	collectable_remaining = collectable_total;
	collectable_attained  = 0;
	collectable_trigger   = false;
}

void ExecuteScript(uint1 script_id)
{
	bool  done = false;
	bool  fires_extinguished = false;
	uint1 x;
	uint1 y;
	uint1 length;

	OBJECT_Type* script;
	OBJECT_Type* object;

	while (!done)
	{
		script = FindObjectEntryById(script_id, false);

		if ((script) && (script->game_piece == GAME_PIECE_Script))
		{
			if (script->flags & FLAG_BONUS)
			{
				bonus_point_delay = 32;
				bonus_point_value = script->game_piece_id * 10;
				
				if (jumpman.x < 80)
					bonus_point_pos.x = jumpman.x + 8;
				else
					bonus_point_pos.x = jumpman.x - 16;
				
				if (jumpman.y < 40)
					bonus_point_pos.y = jumpman.y + 12;
				else
					bonus_point_pos.y = jumpman.y - 12;
				

				game_score += (bonus_point_value - 100);
			}
			else if (script->flags & FLAG_LETTER)
			{
				if (spell_index > 0)
				{
					spell_index--;
					spell_list[spell_index][0] = script->game_piece_id;
				}
			}
			else if (script->flags & FLAG_COMMAND)
			{
				switch (script->command)
				{
				case CMD_Toggle_Girders:
					if (level_toggle_bits & TOGGLE_BIT_Girder)
					{
						ToggleVisibilityObjectType(GAME_PIECE_Girder_Hor, GAME_PIECE_Girder_Down, false);
					}
					level_toggle_bits |= TOGGLE_BIT_Girder;
					break;
				case CMD_Toggle_Rope_Ups:
					if (level_toggle_bits & TOGGLE_BIT_Rope_Up)
					{
						ToggleVisibilityObjectType(GAME_PIECE_Rope_Up, GAME_PIECE_Rope_Up, false);
					}
					level_toggle_bits |= TOGGLE_BIT_Rope_Up;
					break;
				case CMD_Toggle_Rope_Downs:
					if (level_toggle_bits & TOGGLE_BIT_Rope_Down)
					{
						ToggleVisibilityObjectType(TOGGLE_BIT_Rope_Down, TOGGLE_BIT_Rope_Down, false);
					}
					level_toggle_bits |= TOGGLE_BIT_Rope_Down;
					break;
				case CMD_Toggle_Ladders:
					if (level_toggle_bits & TOGGLE_BIT_Ladder)
					{
						ToggleVisibilityObjectType(GAME_PIECE_Ladder, GAME_PIECE_Ladder, false);
					}
					level_toggle_bits |= TOGGLE_BIT_Ladder;
					break;
				case CMD_Toggle_Bombs:
					if (level_toggle_bits & TOGGLE_BIT_Bomb)
					{
						ToggleVisibilityObjectType(GAME_PIECE_Bomb, GAME_PIECE_Bomb, false);
					}
					level_toggle_bits |= TOGGLE_BIT_Bomb;
					break;
				default:
					break;
				}
			}
			else
			{
				object = FindObjectEntryById(script->game_piece_id, true);
	
				if (object)
				{
					if ((object->flags & FLAG_PH) == FLAG_PH)
					{
						PickUpPlatform(object);
					}
					else
					{
						//object->flags |= script->script_flags;
						//if (script->flags & FLAG_TOGGLE)
						//{
						//	object->flags = ((~object->flags) & script->object_flags);
						//}
						//else
						//{
						//	object->flags = script->object_flags;
						//}

						if ((object->game_piece >= GAME_PIECE_Girder_Hor) &&
								(object->game_piece <= GAME_PIECE_Ladder))
						{
							if (script->flags & FLAG_TOGGLE)
							{
								object->flags = ((~object->flags) & script->object_flags);
							}
							else
							{
								object->flags = script->object_flags;
							}
							
							x = object->x * 4;
							y = object->y;
		
							for (length = 0; length < object->length; length++)
							{
								ModifyPlatformPiece(object->game_piece, object->flags, x, y);
		
								if (object->game_piece == GAME_PIECE_Girder_Hor)
								{
									x += 4;
								}
								else if (object->game_piece == GAME_PIECE_Girder_Up)
								{
									x += 4;
									y -= 1;
								}
								else if (object->game_piece == GAME_PIECE_Girder_Down)
								{
									x += 4;
									y += 1;
								}
								else
								{
									y += 4;
								}
							}
						}
						else if (object->game_piece == GAME_PIECE_Fire)
						{
							if (fires_extinguished == false)
							{
								ExtinguishAllFires();
								fires_extinguished = true;
							}

							// try putting this here
							if (script->flags & FLAG_TOGGLE)
							{
								object->flags = ((~object->flags) & script->object_flags);
							}
							else
							{
								object->flags = script->object_flags;
							}

						}
						else if (object->game_piece == GAME_PIECE_Robot_1)
						{
							ENEMY_OBJECT_Type* enemy = FindEnemyObject(object->id);
							
							if (enemy)
							{
								if (script->object_flags & FLAG_HOLD)
								{
									enemy->hold = true;
								}
								else
								{
									enemy->hold = false;
								}			
							}
							
							object->flags = script->object_flags;
						}
						else if (object->game_piece == GAME_PIECE_Follower)
						{
							ReleaseNextEnemy(GAME_PIECE_Follower);
						}
						else if (object->game_piece == GAME_PIECE_Wall)
						{
							ReleaseNextEnemy(GAME_PIECE_Wall);
						}
						//else if (object->game_piece == GAME_PIECE_Freezer)
						//{
						//	ReleaseNextEnemy(GAME_PIECE_Freezer);
						//}
						else if (object->game_piece == GAME_PIECE_Hatchling)
						{
							ReleaseHatchling(object->id);
						}
						else if (object->game_piece == GAME_PIECE_Transporter)
						{
							Teleport(object->id);
						}
						else if (object->game_piece == GAME_PIECE_Door)
						{
							ENEMY_OBJECT_Type* enemy = FindEnemyObject(object->id);
							
							if (enemy)
							{
								if (script->flags & FLAG_OPEN)
								{
									enemy->direction = DIR_Up;
									object->flags &= (~FLAG_TOGGLE);
								}
								else if (script->flags & FLAG_CLOSE)
								{
									enemy->direction = DIR_Down;
									object->flags &= (~FLAG_TOGGLE);
								}			
							}
						}
						else if (object->game_piece == GAME_PIECE_Jumpman)
						{
							if (script->object_flags & FLAG_V)
								jumpman.visible = true;
							else
								jumpman.visible = false;					
						}
						else
						{
							object->flags |= script->object_flags;
						}
						
						
					}

					//GrayUserDrawStr(50 + 4*10,1, "Script", 0, 0);
				}
			}

			script_id = script->script_id;
		}
		else
		{
			done = true;
		}
	}
}

void DrawLetters(void)
{
	if (spell_index < MAX_SPELL_LETTERS)
	{
		int2 i;
	
		//SetUserFont(apogee_font8x8 - 32 * 8, 8, 8);
		SetUserFont(apogee_font8x10 - 32 * 10, 8, 10);

		for (i = spell_index; i < MAX_SPELL_LETTERS; i++)
		{
			if (spell_list[i][0])
			{
				GrayUserDrawStr(76,i*9+1, spell_list[i], COLOR_White, 0);
			}
		}
	
		SetUserFont(font4x7 - 32 * 7, 4, 7);
	}
}

void DrawBonusPoints(void)
{
	if (bonus_point_delay)
	{
		bonus_point_delay--;

		//SetUserFont(apogee_font8x8 - 32 * 8, 8, 8);
		SetUserFont(apogee_font8x10 - 32 * 10, 8, 10);

		GrayUserDrawStr(bonus_point_pos.x,bonus_point_pos.y, "500", bonus_point_delay & 0x3, 0);
		//GrayUserDrawStr(4,4, "!#$%&'()*;+,-./:;<=", COLOR_White, 0);
		//GrayUserDrawStr(4,14, ">?@[\\]^", COLOR_White, 0);
		//GrayUserDrawStr(4,24, "ABCDEFGHIJKLMNOPQRS", COLOR_White, 0);
		//GrayUserDrawStr(4,34, "TUVWXYZ", COLOR_White, 0);
		//GrayUserDrawStr(4,44, "0123456789", COLOR_White, 0);

		SetUserFont(font4x7 - 32 * 7, 4, 7);
	}
}


void ToggleVisibilityObjectType(uint1 game_piece_start, uint1 game_piece_stop, bool restore_visibility)
{
	OBJECT_Type* object;
	bool done = false;
	uint1 object_flags;
	uint1 restore_visible_flag;

	if (restore_visibility)
		restore_visible_flag = FLAG_VA;
	else	
		restore_visible_flag = FLAG_NONE;
		
	ResetIndexByType();
			
	while (!done)
	{
		object = GetNextObjectEntry();

		if (object)
		{
			if ((object->game_piece >= game_piece_start) && (object->game_piece <= game_piece_stop))
			{
				object_flags = object->flags;
				
				//if ((object_flags & FLAG_A) && !(object_flags & FLAG_MOVABLE))
				if (!(object_flags & FLAG_MOVABLE))
				{
					if (object_flags & FLAG_V)
						object_flags &= (~FLAG_VA);
					else
						object_flags |= FLAG_VA;
						
					object_flags |= restore_visible_flag;
					
					if ((object->game_piece >= GAME_PIECE_Girder_Hor) &&
							(object->game_piece <= GAME_PIECE_Ladder))
					{
						ModifyPlatformObject(object, object_flags);
					}
					//else // if (game_piece == GAME_PIECE_Bomb)
					//{
					//	object->flags = object_flags;
					//}
					
					object->flags = object_flags;
				}
			}
		}
		else
		{
			done = true;
		}
	}

}

void ModifyPlatformObject(OBJECT_Type* object, uint1 new_object_flags)
{
	int2 x = object->x * 4;
	int2 y = object->y;
	int2 length;
	
	object->flags = new_object_flags;

	for (length = 0; length < object->length; length++)
	{
		ModifyPlatformPiece(object->game_piece, new_object_flags, x, y);

		if (object->game_piece == GAME_PIECE_Girder_Hor)
		{
			x += 4;
		}
		else if (object->game_piece == GAME_PIECE_Girder_Up)
		{
			x += 4;
			y -= 1;
		}
		else if (object->game_piece == GAME_PIECE_Girder_Down)
		{
			x += 4;
			y += 1;
		}
		else
		{
			y += 4;
		}
	}
}